/*
 * This file is part of PAXLink, which is an AI for OpenTTD
 * Copyright (C) 2009  Leif Linse
 *
 * PAXLink is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * PAXLink is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PAXLink; If not, see <http://www.gnu.org/licenses/> or
 * write to the Free Software Foundation, Inc., 51 Franklin Street, 
 * Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
require("helper.nut");

class OrderList {
	list = null;
	
	constructor()
	{
		this.list = [];
	}
}

function OrderList::AddStop(station_id, flags)
{
	list.append([station_id, flags]);
}

function OrderList::FindOrderDestination(vehicle_id, begin, match_destination)
{
	local num = AIOrder.GetOrderCount(vehicle_id);
	for(local i = begin; i < num; i++)
	{
		if(AIOrder.GetOrderDestination(vehicle_id, i) == match_destination)
			return i;
	}

	return -1;
}

function OrderList::ApplyToVehicle(vehicle_id)
{
	AILog.Info("order list len: " + this.list.len());
	//Helper.BreakPoint(AIVehicle.GetLocation(vehicle_id));

	local i = 0;
	for(; i < list.len(); i++)
	{
		AILog.Info("i = " + i);
		if(i < AIOrder.GetOrderCount(vehicle_id))
		{
			AILog.Info(" i < num orders of vehicle");

			local curr_order_dest_station = AIStation.GetStationID(AIOrder.GetOrderDestination(vehicle_id, i));
			if(curr_order_dest_station == this.list[i][0])
			{
				// current order has correct destination
				AILog.Info("current order has correct destination");
				AIOrder.ChangeOrder(vehicle_id, i, this.list[i][1]); // Make sure the flags are correct
			}
			else
			{
				// current order has wrong destination
				AILog.Info("current order has Wrong destination");

				// First try to see if there is an order with this destination
				local later_order = FindOrderDestination(vehicle_id, i+1, this.list[i][0]);
				if(later_order != -1)
				{
					// Found the destination later in list, move it to i
					AILog.Info("Found correct destination later in list");
					AIOrder.MoveOrder(vehicle_id, later_order, i);
					AIOrder.ChangeOrder(vehicle_id, i, this.list[i][1]); // Make sure the flags are correct
				}
				else
				{
					// There is no order with this destination, make a new order.
					AILog.Info("Did not found correct destination later in list -> insert a new one");
					Helper.SetSign(AIStation.GetLocation(this.list[i][0]), "insert");
					AIOrder.InsertOrder(vehicle_id, i, AIStation.GetLocation(this.list[i][0]), this.list[i][1]);
					AILog.Info("Insert order error msg: " + AIError.GetLastErrorString());
				}
			}
		}
		else
		{
			// the existing order list is shorter than current wanted order.
			AILog.Info("Existing order list is shorter than current i -> Append new order");
			
			// Append wanted order at the end
			Helper.SetSign(AIStation.GetLocation(this.list[i][0]), "append");
			AIOrder.AppendOrder(vehicle_id, AIStation.GetLocation(this.list[i][0]), this.list[i][1]);
			AILog.Info("Append order error msg: " + AIError.GetLastErrorString());
		}

		AILog.Info("Breaking before next loop");
		//Helper.BreakPoint(AIVehicle.GetLocation(vehicle_id));
	}

	AILog.Info("Breaking before removal");
	//Helper.BreakPoint(AIVehicle.GetLocation(vehicle_id));

	// if the vehicle order list contained orders that we don't want to have anymore, they will be located at the end.
	// so remove all orders that are after the last item number in this.list
	local num_to_remove = AIOrder.GetOrderCount(vehicle_id) - this.list.len();
	if(num_to_remove > 0)
	{
		for(local j = 0; j < num_to_remove; j++)
		{
			// Remove the one with id last wanted + 1, since the remaining ones move up after each removal.
			AIOrder.RemoveOrder(vehicle_id, this.list.len());
		}
	}

	AILog.Info("Breaking after removal");
	//Helper.BreakPoint(AIVehicle.GetLocation(vehicle_id));
}
